"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var types_1 = __importDefault(require("./types"));
var utils_1 = require("./utils");
var Op = Object.prototype;
var hasOwn = Op.hasOwnProperty;
function typeAnnotationsPlugin(fork) {
    var _a = fork.use(types_1.default), builders = _a.builders, namedTypes = _a.namedTypes;
    function createTSTypeAnnotator(config) {
        function getTSTypeAnnotation(type) {
            switch (type.kind) {
                case "ArrayType": {
                    var elemTypeAnnotation = getTSTypeAnnotation(type.elemType);
                    // TODO Improve this test.
                    if (namedTypes.TSUnionType.check(elemTypeAnnotation)) {
                        elemTypeAnnotation = builders.tsParenthesizedType(elemTypeAnnotation);
                    }
                    return builders.tsArrayType(elemTypeAnnotation);
                }
                case "IdentityType": {
                    if (type.value === null) {
                        return builders.tsNullKeyword();
                    }
                    switch (typeof type.value) {
                        case "undefined":
                            return builders.tsUndefinedKeyword();
                        case "string":
                            return builders.tsLiteralType(builders.stringLiteral(type.value));
                        case "boolean":
                            return builders.tsLiteralType(builders.booleanLiteral(type.value));
                        case "number":
                            return builders.tsNumberKeyword();
                        case "object":
                            return builders.tsObjectKeyword();
                        case "function":
                            return builders.tsFunctionType();
                        case "symbol":
                            return builders.tsSymbolKeyword();
                        default:
                            return builders.tsAnyKeyword();
                    }
                }
                case "ObjectType": {
                    return builders.tsTypeLiteral.from({
                        members: type.fields.map(function (field) { return getTSPropertySignature(field); }),
                    });
                }
                case "OrType": {
                    return builders.tsUnionType(type.types.map(function (type) { return getTSTypeAnnotation(type); }));
                }
                case "PredicateType": {
                    if (typeof type.name !== "string") {
                        return builders.tsAnyKeyword();
                    }
                    if (hasOwn.call(namedTypes, type.name)) {
                        return config.getReferenceToKind(type.name);
                    }
                    if (/^[$A-Z_][a-z0-9_$]*$/i.test(type.name)) {
                        return builders.tsTypeReference(builders.identifier(type.name));
                    }
                    if (/^number [<>=]+ \d+$/.test(type.name)) {
                        return builders.tsNumberKeyword();
                    }
                    // Not much else to do...
                    return builders.tsAnyKeyword();
                }
                default:
                    return utils_1.assertNever(type);
            }
        }
        function getTSPropertySignature(field) {
            return builders.tsPropertySignature.from({
                key: builders.identifier(field.name),
                typeAnnotation: builders.tsTypeAnnotation(getTSTypeAnnotation(field.type)),
                optional: field.defaultFn != null || field.hidden,
            });
        }
        return {
            getTSTypeAnnotation: getTSTypeAnnotation,
            getTSPropertySignature: getTSPropertySignature,
        };
    }
    return {
        createTSTypeAnnotator: createTSTypeAnnotator,
    };
}
exports.default = typeAnnotationsPlugin;
